<?php
/**
 * ELanguageUrlManager automatically maps the current locale to the created urls and vica versa, with or without using cookie.
 * 
 * I had the inspiration from the great article http://www.yiiframework.com/wiki/294/seo-conform-multilingual-urls-language-selector-widget-i18n/ of c@cba
 * 
 * v1.0
 * @author nlac
 */
class ELanguageUrlManager extends CUrlManager {

/**
 * @param String $languages array of all languages used
 **/
	public $languages = array('en','de','hu');
/**
 * @param String $langKey name of the url param
 **/
	public $langKey = 'lang';
/**
 * @param String $defaultLang language code to be hidden in all urls
 * - if false, all language codes will be forced to be recent in all urls
 **/
	public $defaultLang = false;
/**
 * @param bool $stateful wether to use or not session cookie
 * - once the language is set in the url, the extension tries to maintain the language over the whole session without cookies, so *theoretically* this param can be be false, but set it to true if you have problems with the stateless solution
 **/
	public $stateful = false;
/**
 * @param array $cookieOpts cookie options storing the language
 * - has effect only if $this->stateful is true
 * - check http://www.yiiframework.com/doc/api/1.1/CHttpCookie#__construct-detail
 **/
	public $cookieOpts = array();
/**
 * Regexp matches the base url of the application. 
 *  - anguage code will be inserted right after that
 *  - default: false, in that case it will be Yii:app()->urlManager->baseUrl
 **/
	public $baseUrlPattern = false;

/**
 * @param String $langPattern regexp for the path language part
 * */
	protected $langPattern;





/**
 * Helper method
 **/
	public static function getUrlParams($url) {
		$m = array();$params = array();
		if (preg_match('/\\?(.+)$/', $url, $m) > 0) {		
			$query = explode('&', $m[1]);
			foreach($query as $p)
				if ($p) {
					$p = explode('=', $p); 
					$params[ urldecode($p[0]) ] = @urldecode($p[1]); 
				}
		}
		return $params; 
	}

/**
 * Helper method
 */
	public static function addUrlParams($url, $params=array(), $excludeParams = array()) {
		if (empty($params))
			$params = array();
		$P = self::getUrlParams($url);
		foreach($params as $key=>$val)
			$P[$key] = $val;

		$url = preg_replace('/\\?.*/', '', $url) . '?';
		foreach($P as $key=>$val)
			if (!in_array($key, $excludeParams))
				$url .= ($key . '=' . preg_replace('/%(25)+/','%',urlencode($val)) . '&');
		return rtrim($url,'&');
	}

	
/**
 * Helper method
 **/
	public static function removeUrlParams($url, $params=array()) {
		if (empty($params))
			$params = array();
		$P = self::getUrlParams($url);
		foreach($params as $val)
			unset($P[$val]);
		
		$url = preg_replace('/\\?.*/', '', $url) . '?';
		foreach($P as $key=>$val)
			$url .= ($key . '=' . preg_replace('/%(25)+/','%',urlencode($val)) . '&');
		return rtrim($url,'&');
	}





/**
 * Forces all urls to contain the language code
 */
	public function createUrl($route, $params=array(), $ampersand='&') {
		
		if ($this->defaultLang == Yii::app()->language) {
			if (is_array($params) && isset($params[$this->langKey]))
				unset($params[$this->langKey]);
			return parent::createUrl($route, $params, $ampersand);
		}
		
		return parent::createUrl($route, array_merge(array(
			$this->langKey => Yii::app()->language
		), $params), $ampersand);
	}

/**
 * Injects the language part into an existing url
 * - if url is not set, the current one will be transformed
 * - array-format urls won't be touched
 * 
 * - check redirect() method in ELanguageController.php
 */
	public function createStrUrl($url=null, $lang=null) {
		if (is_array($url))
			return $url;
		
		if (!$url)
			$url = Yii::app()->request->requestUri;
		if (!$lang)
			$lang = Yii::app()->language;
		
		//$baseUrl = preg_replace('#/+$#', '', $r->baseUrl);
		if ($this->urlFormat == 'path') {
			return preg_replace('@((' . $this->baseUrlPattern . ')(/\w+\.php)?)(/' . $this->langPattern . ')?@i', 
				'$1' . ($this->defaultLang==$lang ? '' : '/' . $lang), 
				$url);		
		}
		
		//"get" case
		if ($this->defaultLang==$lang) {
			return self::removeUrlParams($url, array($this->langKey));
		}
		
		return self::addUrlParams($url, array($this->langKey => $lang));
	}

/**
 * Creates an array-format rule
 */
	protected function makeLangRule($pattern, $route, $defaultLangParam = false) {
		
		if (is_string($route))
			$route = array($route, 'pattern'=>$pattern);
		
		if ($defaultLangParam) {
			if (!isset($route['defaultParams']))
				$route['defaultParams'] = array();
			$route['defaultParams'] = array_merge($route['defaultParams'], 
				array($this->langKey => $defaultLangParam));
		}

		return $route;
	}
	
/**
 * Transforms all rules to the new url format + adds some default rules
 */
	public function init() {

		if ($this->baseUrlPattern===false)
			$this->baseUrlPattern = preg_replace('#/+$#', '', $this->baseUrl);
		
		//Transforming all configured rules to include the language + adding default rules
		
		//regex matches all lang codes
		$this->langPattern = '(' . implode('|', $this->languages) . ')';
		
		//regex do not matches any lang codes
		$noLangPattern = '((?!' . $this->langPattern . ')\w*)|(' . $this->langPattern . '\w+)';

		$pre = '<' . $this->langKey . ':' . $this->langPattern . '>';
		$newRules = array();
		
		//getting all module ids
		$moduleIds = array();
		foreach(Yii::app()->modules as $id=>$m) {
			$moduleIds[] = $id;
		}
		if (!empty($moduleIds))
			$moduleIds = '(' . implode('|', $moduleIds) . ')';
		
		//adding default rules to the end of the rule set
		if (!empty($moduleIds)) {
			//mapping module urls
			$this->rules['<m:' . $moduleIds . '>/<c:\w+>/<a:\w+>/*'] = '<m>/<c>/<a>';
			$this->rules['<m:' . $moduleIds . '>/<c:\w+>/*'] = '<m>/<c>';
			$this->rules['<m:' . $moduleIds . '>/*'] = '<m>';
		}
		//mapping /controller/action urls
		$this->rules['<c:' . $noLangPattern . '>/<a:\w+>/*'] = '<c>/<a>';
		//mapping /controller urls
		$this->rules['<c:' . $noLangPattern . '>/*'] = '<c>';
		//mapping default urls (/, /en, /hu etc)
		$this->rules['/*'] = '';
		
		//preparin all rules to do the lang. mapping
		foreach($this->rules as $pattern=>$route) {
			//if a default lang is set (hidden in the urls), add an extra rule with defaultParams 
			if ($this->defaultLang)
				$newRules[] = $this->makeLangRule($pattern, $route, $this->defaultLang);
			//pattern prepended with the language param
			$newRules[] = $this->makeLangRule($pre . ($pattern ? '/' . $pattern : ''), $route, false);
		}
		
		//updating rules
		$this->rules = $newRules;
		
		//uncomment it to check your final rules generated
		//Yii::log('generated rules: ' . htmlentities( print_r($this->rules,true)), 'info');
		parent::init();
	}

/**
 * Sets the current language from request/cookie
 * - check init() method in ELanguageController.php
 */
	public function setLang() {
		if (@$_GET[$this->langKey])
			Yii::app()->language = strtolower($_GET[$this->langKey]);
		else if ($this->stateful) {
			if (isset(Yii::app()->request->cookies[$this->langKey]))
				Yii::app()->language = Yii::app()->request->cookies[$this->langKey]->value;
		}

		if ($this->stateful)
			Yii::app()->request->cookies[$this->langKey] = new CHttpCookie($this->langKey, Yii::app()->language, $this->cookieOpts);
	}

/**
 * Example language selector view
 */
	public function demoLangSelector() {
		$curlang = Yii::app()->language;

		echo '<div class="langselector">';
		foreach(Yii::app()->urlManager->languages as $lang) {
			if ($lang == $curlang) {
				echo '<span class="icon icon-' . $lang . ' icon-active">' . $lang . '</span> ';
			} else {
				$href = Yii::app()->urlManager->createStrUrl(null,$lang);
				echo '<a class="icon icon-' . $lang . '" href="' . $href . '">' . $lang . '</a> ';
			}
		}

		echo '</div>';
	}

}
